package ru.dbotthepony.mc.otm.capability;

import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.food.FoodData;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fmllegacy.network.PacketDistributor;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.network.AndroidCapabilityChangePacket;
import ru.dbotthepony.mc.otm.network.MatteryNetwork;

import java.math.BigDecimal;
import java.util.Random;

public class AndroidCapabilityPlayer extends AndroidCapability {
	public static final ResourceLocation ANDROID_CAPABILITY_LOCATION = new ResourceLocation(OverdriveThatMatters.MOD_ID, "android_capability");

	@SubscribeEvent
	public static void onAttachCapabilityEvent(AttachCapabilitiesEvent<Entity> event) {
		if (event.getObject() instanceof Player ply) {
			event.addCapability(ANDROID_CAPABILITY_LOCATION, new AndroidCapabilityPlayer(ply));
		}
	}

	@SubscribeEvent
	public static void onPlayerCloneEvent(PlayerEvent.Clone event) {
		event.getPlayer().getCapability(MatteryCapability.ANDROID).ifPresent((cap) -> {
			LazyOptional<IAndroidCapability> resolver = event.getOriginal().getCapability(MatteryCapability.ANDROID);

			if (!resolver.isPresent() || resolver.resolve().isEmpty()) {
				event.getOriginal().reviveCaps();
				resolver = event.getOriginal().getCapability(MatteryCapability.ANDROID);
			}

			if (!resolver.isPresent() || resolver.resolve().isEmpty()) {
				event.getOriginal().invalidateCaps();
				return;
			}

			cap.deserializeNBT(resolver.resolve().get().serializeNBT());
			event.getOriginal().invalidateCaps();
		});
	}

	private final Player ply;

	public AndroidCapabilityPlayer(Player ent) {
		super(ent);
		ply = ent;
	}

	@Override
	protected void tickNetwork() {
		AndroidCapabilityChangePacket packet = writeChanges();

		if (packet != null)
			MatteryNetwork.ANDROID_PLAYER.send(PacketDistributor.PLAYER.with(() -> (ServerPlayer) ent), packet);
	}

	@Override
	public void tickInner() {
		super.tickInner();

		// TODO: Maybe passive drain?
		// extractEnergyInner(BigDecimal.valueOf(new Random().nextDouble()), false);

		FoodData stats = ply.getFoodData();

		if (stats.needsFood()) {
			int food_points_missing = 20 - stats.getFoodLevel();
			boolean food_changed = false;
			float saturation_points_missing = 20F - stats.getSaturationLevel();
			boolean saturation_changed = false;

			while (food_points_missing > 0) {
				if (this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true).equals(ENERGY_FOR_HUNGER_POINT)) {
					this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
					food_points_missing--;
					food_changed = true;
				} else {
					break;
				}
			}

			while (saturation_points_missing >= 1F) {
				if (this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true).equals(ENERGY_FOR_HUNGER_POINT)) {
					this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
					saturation_points_missing--;
					saturation_changed = true;
				} else {
					break;
				}
			}

			if (food_changed) {
				stats.setFoodLevel(20 - food_points_missing);
			}

			if (saturation_changed) {
				stats.setSaturation(20F - saturation_points_missing);
			}
		}
	}
}
