package ru.dbotthepony.mc.otm.menu.data;

import net.minecraft.world.inventory.ContainerData;

import java.math.BigDecimal;
import java.math.BigInteger;

public class BigDecimalDataContainer implements ContainerData {
	public static final int NETWORK_PAYLOAD_SIZE = 8;

	private BigDecimal value;
	private final int[] buffer = new int[NETWORK_PAYLOAD_SIZE + 1];

	public BigDecimal getDecimal() {
		if (value != null) {
			return value;
		}

		byte[] _build = new byte[NETWORK_PAYLOAD_SIZE * 4];

		int build_index = 0;

		// read payload
		for (int i = 1; i <= NETWORK_PAYLOAD_SIZE; i++) {
			_build[build_index] = (byte) ((buffer[i] & 0xFF000000) >> 24);
			_build[build_index + 1] = (byte) ((buffer[i] & 0xFF0000) >> 16);
			_build[build_index + 2] = (byte) ((buffer[i] & 0xFF00) >> 8);
			_build[build_index + 3] = (byte) (buffer[i] & 0xFF);

			build_index += 4;
		}

		byte[] build = new byte[_build[0] & 0xFF];

		if (_build[0] != 0) {
			System.arraycopy(_build, 1, build, 0, build.length);

			value = new BigDecimal(new BigInteger(build), buffer[0]);
			return value;
		}

		value = new BigDecimal(0);
		return value;
	}

	public void setDecimal(BigDecimal decimal) {
		if (decimal.equals(value)) {
			return;
		}

		value = decimal;

		int scale = decimal.scale();
		BigInteger integer = decimal.unscaledValue();

		buffer[0] = scale;
		byte[] _build = integer.toByteArray();
		byte[] build = new byte[NETWORK_PAYLOAD_SIZE * 4];

		// insert, shift by one
		System.arraycopy(_build, 0, build, 1, Math.min(_build.length, NETWORK_PAYLOAD_SIZE * 4 - 1));

		// tell how many bytes are in there
		build[0] = (byte) _build.length;

		int build_index = 0;

		// write
		for (int i = 1; i <= NETWORK_PAYLOAD_SIZE; i++) {
			buffer[i] = (build[build_index] & 0xFF) << 24 |
					(build[build_index + 1] & 0xFF) << 16 |
					(build[build_index + 2] & 0xFF) << 8 |
					(build[build_index + 3] & 0xFF);

			build_index += 4;
		}
	}

	// override
	protected void updateValue() {

	}

	@Override
	public int get(int index) {
		updateValue();
		return buffer[index];
	}

	@Override
	public void set(int index, int _value) {
		if (buffer[index] == _value)
			return;

		value = null;
		buffer[index] = _value;
	}

	@Override
	public int getCount() {
		return NETWORK_PAYLOAD_SIZE + 1;
	}
}
