package ru.dbotthepony.mc.otm.network;

import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.fml.DistExecutor;
import net.minecraftforge.fmllegacy.network.NetworkEvent;
import net.minecraftforge.fmllegacy.network.PacketDistributor;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.function.Supplier;

public class AndroidCapabilityChangePacket {
	public enum ChangeType {
		TERMINATION,
		ENERGY_CHANGED,
		MAX_ENERGY_CHANGED,
		BATTERY_STACK_CHANGED,
	}

	public static ChangeType[] CHANGE_MAP = new ChangeType[] {
		ChangeType.TERMINATION,
		ChangeType.ENERGY_CHANGED,
		ChangeType.MAX_ENERGY_CHANGED,
		ChangeType.BATTERY_STACK_CHANGED,
	};

	public BigDecimal energy;
	public BigDecimal max_energy;
	public ItemStack battery;

	public boolean isEmpty() {
		return energy == null && max_energy == null && battery == null;
	}

	// build state
	public void energyChanged(BigDecimal energy) {
		this.energy = energy;
	}

	public void maxEnergyChanged(BigDecimal energy) {
		this.max_energy = energy;
	}

	public void batteryStackChanged(ItemStack stack) {
		this.battery = stack;
	}

	// encode state
	public void encodeNetwork(FriendlyByteBuf buffer) {
		if (energy != null) {
			buffer.writeByte(ChangeType.ENERGY_CHANGED.ordinal());
			NetworkHelper.writeDecimal(buffer, energy);
		}

		if (max_energy != null) {
			buffer.writeByte(ChangeType.MAX_ENERGY_CHANGED.ordinal());
			NetworkHelper.writeDecimal(buffer, max_energy);
		}

		if (battery != null) {
			buffer.writeByte(ChangeType.BATTERY_STACK_CHANGED.ordinal());
			buffer.writeItem(battery);
		}

		buffer.writeByte(ChangeType.TERMINATION.ordinal());
	}

	// handle state
	public void handleMessage(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);

		context.get().enqueueWork(() -> {
			DistExecutor.safeRunWhenOn(Dist.CLIENT, () -> () -> ClientAndroidCapabilityChangePacketHandler.handlePacket(this, context));
		});
	}

	// decode state
	public static AndroidCapabilityChangePacket decodeNetwork(FriendlyByteBuf buffer) {
		AndroidCapabilityChangePacket packet = new AndroidCapabilityChangePacket();

		ChangeType read = CHANGE_MAP[buffer.readByte()];

		while (read != ChangeType.TERMINATION) {
			switch (read) {
				case ENERGY_CHANGED -> packet.energy = NetworkHelper.readDecimal(buffer);
				case MAX_ENERGY_CHANGED -> packet.max_energy = NetworkHelper.readDecimal(buffer);
				case BATTERY_STACK_CHANGED -> packet.battery = buffer.readItem();
			}

			read = CHANGE_MAP[buffer.readByte()];
		}

		return packet;
	}
}
